@@ -132,11 +132,12 @@ FileInfo::FileInfo(const QString &name, const std::initializer_list<FileInfo> &c
         addChild(source);
 }
 
-void FileInfo::addChild(const FileInfo &info)
+FileInfo &FileInfo::addChild(const FileInfo &info)
 {
-    auto &dest = this->children[info.name] = info;
+    FileInfo &dest = this->children[info.name] = info;
     dest.parentPath = path();
     dest.fixupParentPathRecursively();
+    return dest;
 }
 
 void FileInfo::remove(const QString &relativePath)
@@ -165,7 +166,10 @@ void FileInfo::appendByte(const QString &relativePath, char contentChar)
     Q_UNUSED(contentChar);
     FileInfo *file = findInvalidatingEtags(relativePath);
     Q_ASSERT(file);
-    file->size += 1;
+    if (!file->isDehydratedPlaceholder) {
+        file->contentSize += 1;
+    }
+    file->fileSize += 1;
 }
 
 void FileInfo::modifyByte(const QString &relativePath, quint64 offset, char contentChar)
@@ -202,7 +206,7 @@ void FileInfo::setModTime(const QString &relativePath, const QDateTime &modTime)
 {
     FileInfo *file = findInvalidatingEtags(relativePath);
     Q_ASSERT(file);
-    file->lastModified = modTime;
+    file->setLastModified(modTime);
 }
 
 FileInfo *FileInfo::find(PathComponents pathComponents, const bool invalidateEtags)
@@ -249,14 +253,45 @@ FileInfo *FileInfo::create(const QString &relativePath, qint64 size, char conten
     return &child;
 }
 
-bool FileInfo::operator==(const FileInfo &other) const
+bool FileInfo::equals(const FileInfo &other, CompareWhat compareWhat) const
 {
-    // Consider files to be equal between local<->remote as a user would.
-    return name == other.name
-        && isDir == other.isDir
-        && size == other.size
-        && contentChar == other.contentChar
-        && children == other.children;
+    // Only check the content and contentSize if both files are hydrated:
+    if (!isDehydratedPlaceholder && !other.isDehydratedPlaceholder) {
+        if (contentSize != other.contentSize || contentChar != other.contentChar) {
+            return false;
+        }
+    }
+
+    // We need to check this before we use isDir in the next if-statement:
+    if (isDir != other.isDir) {
+        return false;
+    }
+
+    if (compareWhat == CompareLastModified) {
+        // Don't check directory mtime: it might change when (unsynced) files get created.
+        if (!isDir && _lastModifiedInSecondsUTC != other._lastModifiedInSecondsUTC) {
+            return false;
+        }
+    }
+
+    if (name != other.name || fileSize != other.fileSize) {
+        return false;
+    }
+
+    if (children.size() != other.children.size()) {
+        return false;
+    }
+
+    for (auto it = children.constBegin(), eit = children.constEnd(); it != eit; ++it) {
+        auto oit = other.children.constFind(it.key());
+        if (oit == other.children.constEnd()) {
+            return false;
+        } else if (!it.value().equals(oit.value(), compareWhat)) {
+            return false;
+        }
+    }
+
+    return true;
 }
 
 QString FileInfo::path() const
@@ -329,10 +364,10 @@ FakePropfindReply::FakePropfindReply(FileInfo &remoteRootFileInfo, QNetworkAcces
         } else
             xml.writeEmptyElement(davUri, QStringLiteral("resourcetype"));
 
-        auto gmtDate = fileInfo.lastModified.toUTC();
+        auto gmtDate = fileInfo.lastModifiedInUtc();
         auto stringDate = QLocale::c().toString(gmtDate, QStringLiteral("ddd, dd MMM yyyy HH:mm:ss 'GMT'"));
         xml.writeTextElement(davUri, QStringLiteral("getlastmodified"), stringDate);
-        xml.writeTextElement(davUri, QStringLiteral("getcontentlength"), QString::number(fileInfo.size));
+        xml.writeTextElement(davUri, QStringLiteral("getcontentlength"), QString::number(fileInfo.contentSize));
         xml.writeTextElement(davUri, QStringLiteral("getetag"), QStringLiteral("\"%1\"").arg(QString::fromLatin1(fileInfo.etag)));
         xml.writeTextElement(ocUri, QStringLiteral("permissions"), !fileInfo.permissions.isNull() ? QString(fileInfo.permissions.toString()) : fileInfo.isShared ? QStringLiteral("SRDNVCKW")
                                                                                                                                                                  : QStringLiteral("RDNVCKW"));
@@ -409,20 +444,21 @@ FileInfo *FakePutReply::perform(FileInfo &remoteRootFileInfo, const QNetworkRequ
     Q_ASSERT(!fileName.isEmpty());
     FileInfo *fileInfo = remoteRootFileInfo.find(fileName);
     if (fileInfo) {
-        fileInfo->size = putPayload.size();
+        fileInfo->contentSize = putPayload.size();
         fileInfo->contentChar = putPayload.at(0);
     } else {
         // Assume that the file is filled with the same character
         fileInfo = remoteRootFileInfo.create(fileName, putPayload.size(), putPayload.at(0));
     }
-    fileInfo->lastModified = OCC::Utility::qDateTimeFromTime_t(request.rawHeader("X-OC-Mtime").toLongLong());
+    fileInfo->fileSize = fileInfo->contentSize; // it's hydrated on the server, so these are the same
+    fileInfo->setLastModifiedFromSecondsUTC(request.rawHeader("X-OC-Mtime").toLongLong());
     remoteRootFileInfo.find(fileName, /*invalidate_etags=*/true);
     return fileInfo;
 }
 
 void FakePutReply::respond()
 {
-    emit uploadProgress(fileInfo->size, fileInfo->size);
+    emit uploadProgress(fileInfo->contentSize, fileInfo->contentSize);
     setRawHeader("OC-ETag", fileInfo->etag);
     setRawHeader("ETag", fileInfo->etag);
     setRawHeader("OC-FileID", fileInfo->fileId);
@@ -536,12 +572,13 @@ void FakeGetReply::respond()
         return;
     }
     payload = fileInfo->contentChar;
-    size = fileInfo->size;
+    size = fileInfo->contentSize;
     setHeader(QNetworkRequest::ContentLengthHeader, size);
     setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
     setRawHeader("OC-ETag", fileInfo->etag);
     setRawHeader("ETag", fileInfo->etag);
     setRawHeader("OC-FileId", fileInfo->fileId);
+    setRawHeader("X-OC-Mtime", QByteArray::number(fileInfo->lastModifiedInSecondsUTC()));
     emit metaDataChanged();
     if (bytesAvailable())
         emit readyRead();
@@ -609,6 +646,7 @@ void FakeGetWithDataReply::respond()
     setRawHeader("OC-ETag", fileInfo->etag);
     setRawHeader("ETag", fileInfo->etag);
     setRawHeader("OC-FileId", fileInfo->fileId);
+    setRawHeader("X-OC-Mtime", QByteArray::number(fileInfo->lastModifiedInSecondsUTC()));
     emit metaDataChanged();
     if (bytesAvailable())
         emit readyRead();
@@ -677,12 +715,12 @@ FileInfo *FakeChunkMoveReply::perform(FileInfo &uploadsFileInfo, FileInfo &remot
         if (chunkNameLongLong != prev)
             break;
         Q_ASSERT(!x.isDir);
-        Q_ASSERT(x.size > 0); // There should not be empty chunks
-        size += x.size;
+        Q_ASSERT(x.contentSize > 0); // There should not be empty chunks
+        size += x.contentSize;
         Q_ASSERT(!payload || payload == x.contentChar);
         payload = x.contentChar;
         ++count;
-        prev = chunkNameLongLong + x.size;
+        prev = chunkNameLongLong + x.contentSize;
     }
     Q_ASSERT(sourceFolderChildren.count() == count); // There should not be holes or extra files
 
@@ -699,14 +737,15 @@ FileInfo *FakeChunkMoveReply::perform(FileInfo &uploadsFileInfo, FileInfo &remot
         if (request.rawHeader("If") != start + " ([\"" + fileInfo->etag + "\"])") {
             return nullptr;
         }
-        fileInfo->size = size;
+        fileInfo->contentSize = size;
         fileInfo->contentChar = payload;
+        fileInfo->fileSize = fileInfo->contentSize; // it's hydrated on the server, so these are the same
     } else {
         Q_ASSERT(!request.hasRawHeader("If"));
         // Assume that the file is filled with the same character
         fileInfo = remoteRootFileInfo.create(fileName, size, payload);
     }
-    fileInfo->lastModified = OCC::Utility::qDateTimeFromTime_t(request.rawHeader("X-OC-Mtime").toLongLong());
+    fileInfo->setLastModifiedFromSecondsUTC(request.rawHeader("X-OC-Mtime").toLongLong());
     remoteRootFileInfo.find(fileName, /*invalidate_etags=*/true);
 
     return fileInfo;
@@ -881,8 +920,9 @@ QNetworkReply *FakeQNAM::createRequest(QNetworkAccessManager::Operation op, cons
     return reply;
 }
 
-FakeFolder::FakeFolder(const FileInfo &fileTemplate)
+FakeFolder::FakeFolder(const FileInfo &fileTemplate, OCC::Vfs::Mode vfsMode)
     : _localModifier(_tempDir.path())
+    , _vfsMode(vfsMode)
 {
     // Needs to be done once
     OCC::SyncEngine::minimumFileAgeForUpload = std::chrono::milliseconds(0);
@@ -935,13 +975,23 @@ void FakeFolder::switchToVfs(QSharedPointer<OCC::Vfs> vfs)
     vfsParams.account = _account;
     vfsParams.journal = _journalDb.get();
     vfsParams.providerName = QStringLiteral("OC-TEST");
+    vfsParams.providerDisplayName = QStringLiteral("OC-TEST");
     vfsParams.providerVersion = QStringLiteral("0.1");
     QObject::connect(_syncEngine.get(), &QObject::destroyed, vfs.data(), [vfs]() {
         vfs->stop();
         vfs->unregisterFolder();
     });
 
+    QObject::connect(vfs.get(), &OCC::Vfs::error, vfs.get(), [](const QString &error) {
+        QFAIL(qUtf8Printable(error));
+    });
+    QSignalSpy spy(vfs.get(), &OCC::Vfs::started);
     vfs->start(vfsParams);
+
+    // don't use QVERIFY outside of the test slot
+    if (spy.isEmpty() && !spy.wait()) {
+        QFAIL("VFS Setup failed");
+    }
 }
 
 FileInfo FakeFolder::currentLocalState()
@@ -990,6 +1040,11 @@ void FakeFolder::execUntilItemCompleted(const QString &relativePath)
     QVERIFY(false);
 }
 
+bool FakeFolder::isDehydratedPlaceholder(const QString &filePath)
+{
+    return _syncEngine->syncOptions()._vfs->isDehydratedPlaceholder(filePath);
+}
+
 void FakeFolder::toDisk(QDir &dir, const FileInfo &templateFi)
 {
     for (const auto &child : templateFi.children) {
@@ -1001,9 +1056,9 @@ void FakeFolder::toDisk(QDir &dir, const FileInfo &templateFi)
         } else {
             QFile file { dir.filePath(child.name) };
             file.open(QFile::WriteOnly);
-            file.write(QByteArray {}.fill(child.contentChar, child.size));
+            file.write(QByteArray {}.fill(child.contentChar, child.contentSize));
             file.close();
-            OCC::FileSystem::setModTime(file.fileName(), OCC::Utility::qDateTimeToTime_t(child.lastModified));
+            OCC::FileSystem::setModTime(file.fileName(), child.lastModifiedInSecondsUTC());
         }
     }
 }
@@ -1016,17 +1071,29 @@ void FakeFolder::fromDisk(QDir &dir, FileInfo &templateFi)
             QDir subDir = dir;
             subDir.cd(diskChild.fileName());
             FileInfo &subFi = templateFi.children[diskChild.fileName()] = FileInfo { diskChild.fileName() };
+            subFi.setLastModified(diskChild.lastModified());
             fromDisk(subDir, subFi);
         } else {
-            QFile f { diskChild.filePath() };
-            f.open(QFile::ReadOnly);
-            auto content = f.read(1);
-            if (content.size() == 0) {
-                qWarning() << "Empty file at:" << diskChild.filePath();
-                continue;
+            FileInfo fi(diskChild.fileName());
+            fi.isDir = false;
+            fi.fileSize = diskChild.size();
+            fi.isDehydratedPlaceholder = isDehydratedPlaceholder(diskChild.absoluteFilePath());
+            fi.setLastModified(diskChild.lastModified());
+            if (fi.isDehydratedPlaceholder) {
+                fi.contentChar = '\0';
+            } else {
+                QFile f { diskChild.filePath() };
+                f.open(QFile::ReadOnly);
+                auto content = f.read(1);
+                if (content.size() == 0) {
+                    qWarning() << "Empty file at:" << diskChild.filePath();
+                    continue;
+                }
+                fi.contentChar = content.at(0);
+                fi.contentSize = fi.fileSize;
             }
-            char contentChar = content.at(0);
-            templateFi.children.insert(diskChild.fileName(), FileInfo { diskChild.fileName(), diskChild.size(), contentChar });
+
+            templateFi.children.insert(fi.name, fi);
         }
     }
 }
@@ -1055,11 +1122,11 @@ FileInfo FakeFolder::dbState() const
         auto &item = parentDir.children[name];
         item.name = name;
         item.parentPath = parentDir.path();
-        item.size = record._fileSize;
+        item.contentSize = record._fileSize;
         item.isDir = record._type == ItemTypeDirectory;
         item.permissions = record._remotePerm;
         item.etag = record._etag;
-        item.lastModified = OCC::Utility::qDateTimeFromTime_t(record._modtime);
+        item.setLastModifiedFromSecondsUTC(record._modtime);
         item.fileId = record._fileId;
         item.checksums = record._checksumHeader;
         // item.contentChar can't be set from the db
